@@ -281,10 +281,10 @@ gst_ogm_audio_parse_base_init (GstOgmParseClass * klass)
       "parse an OGM audio header and stream",
       "GStreamer maintainers <gstreamer-devel@lists.freedesktop.org>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_factory_audio));
-  audio_src_templ = gst_pad_template_new ("src",
-      GST_PAD_SRC, GST_PAD_SOMETIMES, caps);
+  gst_element_class_add_static_pad_template (element_class,
+      &sink_factory_audio);
+  audio_src_templ =
+      gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_SOMETIMES, caps);
   gst_element_class_add_pad_template (element_class, audio_src_templ);
   gst_caps_unref (caps);
 }
@@ -300,10 +300,10 @@ gst_ogm_video_parse_base_init (GstOgmParseClass * klass)
       "parse an OGM video header and stream",
       "GStreamer maintainers <gstreamer-devel@lists.freedesktop.org>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_factory_video));
-  video_src_templ = gst_pad_template_new ("src",
-      GST_PAD_SRC, GST_PAD_SOMETIMES, caps);
+  gst_element_class_add_static_pad_template (element_class,
+      &sink_factory_video);
+  video_src_templ =
+      gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_SOMETIMES, caps);
   gst_element_class_add_pad_template (element_class, video_src_templ);
   gst_caps_unref (caps);
 }
@@ -320,8 +320,7 @@ gst_ogm_text_parse_base_init (GstOgmParseClass * klass)
       "parse an OGM text header and stream",
       "GStreamer maintainers <gstreamer-devel@lists.freedesktop.org>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_factory_text));
+  gst_element_class_add_static_pad_template (element_class, &sink_factory_text);
   text_src_templ = gst_pad_template_new ("src",
       GST_PAD_SRC, GST_PAD_SOMETIMES, caps);
   gst_element_class_add_pad_template (element_class, text_src_templ);
