@@ -47,6 +47,9 @@ struct _GstStreamSynchronizer
   /* < private > */
   GMutex lock;
   gboolean shutdown;
+  gboolean send_gap_event;
+  gboolean eos;
+  gboolean flushing;
 
   GList *streams;
   guint current_stream_number;
